from enum import Enum


class Region(Enum):
    Belgium = "Belgium"
    Brussels = "Brussels"
    Flanders = "Flanders"
    Wallonia = "Wallonia"
    Antwerp = "Antwerp"
    East_Flanders = "East-Flanders"
    Flemish_Brabant = "Flemish-Brabant"
    Limburg = "Limburg"
    West_Flanders = "West-Flanders"
    Hainaut = "Hainaut"
    Liège = "Liège"
    Luxembourg = "Luxembourg"
    Namur = "Namur"
    Walloon_Brabant = "Walloon-Brabant"
